--[[ 
CLibBox
- Lib to Create little Yellow Box

Create : 
CLibBox:CreateBox(  ....)
returns the frame with injected Methods

Change
CLibBox:MakeBoxDragable(box, callback,  ...)


Injected Functions
SetSize(width, height)
SetPos(x,y)   -- absolute Position
SetText(text)  -- box text
Show()
Hide()
SetBorderVisible(true/false)  -- enable the thin border

-- If Draging enabled lock/unlock moving
Lock()   
Unlock()

]]

local version = 1;

--exit if newer version found
if (CLibBox) and (CLibBox.version >= version) then 
	return
end;

--base Lib
CLibBox = {};
CLibBox.version = version;

--functions to add to the frame
CLibBox.injects = {};


--Create a New Box Frame, returns the new createt frame
--Arguments
-- name - name of the Frame
-- parent - Frame Parent(optional)
-- type - Frame type (optional)
--template  - Frame Template (optional)
function CLibBox:CreateBox(name, parent, type, template )
	
	if not name then
		error("CLibBox:Create - Name Required!");
	end;
		
	if not type then
		type = "Frame"
	end;
	if not template then
		template = "CLibBox_Box_Template"
	end;
	
	if not parent then
		parent = UIParent
	end;

	local f = CreateFrame(type, name, UIParent, template);
	
	
	f.textFrame = getglobal(name.."_Text");
	
	for key, value in pairs(CLibBox.injects) do
		f[key] = value;
	end;
		
	return f;
end

--[[
Make an existing box dragable 
will execute callback(x, y) at end of dragging
 "..." are the buttons to register draging for ]]
function CLibBox:MakeBoxDragable(box, callback, ...)
	if not box then
		error ("CLibBox:MakeDragable - box Required!"); 
	end;		
	
	box.DragEnd = callback;
	box:SetScript("OnDragStart", box.DragStart);
	box:SetScript("OnDragStop", box.DragStop);
	
	box:RegisterForDrag(...);
end;

function CLibBox.injects:SetText(text)	
	self.textFrame:SetText(text);
end

function CLibBox.injects:GetText()	
	return self.textFrame:GetText();
end


function CLibBox.injects:Unlock()
	self.unlocked = true;
end;

function CLibBox.injects:Lock()
	self.unlocked = nil;
end;

function CLibBox.injects:SetPos(x, y)	
	if (x) and (y) then
		self:ClearAllPoints();
		self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
	end;
end;


function CLibBox.injects:SetSize(width, height)
	local border;
	local name = self:GetName();
	if (width) then
		self:SetWidth(width);
		border = getglobal(name.."_1");
		border:SetWidth(width);
		border = getglobal(name.."_2");
		border:SetWidth(width);
	end;
	if (height) then
		self:SetHeight(height);
		border = getglobal(name.."_3");
		border:SetHeight(height);
		border = getglobal(name.."_4");
		border:SetHeight(height);
	end;
end;

function CLibBox.injects:DragStart()	
	if (self.unlocked) then
		self:StartMoving();
	end;
end

function CLibBox.injects:DragStop()	
	self:StopMovingOrSizing();
	if (self.DragEnd) then
		local _, _, _, x, y = self:GetPoint()
		self.DragEnd(x, y);
	end;
end

function CLibBox.injects:SetBorderVisible(check)
	local border;
	local name = self:GetName();
	if (check) then
		border = getglobal(name.."_1");
		border:Show();	
		border = getglobal(name.."_2");
		border:Show();	
		border = getglobal(name.."_3");
		border:Show();	
		border = getglobal(name.."_4");
		border:Show();	
	else
		border = getglobal(name.."_1");
		border:Hide();	
		border = getglobal(name.."_2");
		border:Hide();	
		border = getglobal(name.."_3");
		border:Hide();	
		border = getglobal(name.."_4");
		border:Hide();	
	end;
end;