-- CPala Core, Basic Stuff Loading, Chat Command Handling, and Shared Events

--[[
Events : 
CPALA_UPDATE_LIST Fired when buffList or needList was updatet
CPALA_UPDATE_SUMMARY Fired when count of needs/buffs/faded update
arg1 = needs
arg2 = buffs
arg3 = faded
arg4 = seal  (string)
]]

--Ace Addon
CPala = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceHook-2.1")

local roster = AceLibrary("Roster-2.1"); -- helper


--Keybinding Text
BINDING_HEADER_CPALA = "CPala Bindings";


--Used Strings for Specials States
CPala.selfbuff = "SelfBuff";
CPala.none = "NONE";

--*****Database*******
CPala:RegisterDB("CPalaDB", "CPalaPC")
CPala:RegisterDefaults("char", {
	-- Need List Options
	defaultBlessings = {
		["WARRIOR"] = "Greater Blessing of Might",
		["PALADIN"] = "Greater Blessing of Might",
		["MAGE"] = "Greater Blessing of Salvation",
		["HUNTER"] = "Greater Blessing of Might",
		["SHAMAN"] = "Greater Blessing of Wisdom",
		["PRIEST"] = "Greater Blessing of Wisdom",
		["ROGUE"] = "Greater Blessing of Might",
		["DRUID"] = "Greater Blessing of Wisdom",
		["WARLOCK"] = "Greater Blessing of Salvation",		
		
		["PET"] = "NONE",   -- test
		[CPala.selfbuff] = "Righteous Fury",
	},
	--default Blessing Templates ...
	templates = nil,  --nil by default becacuse default is set at first load	
	-- Box Options
	buffTime = {
		enabled = true,
		shorts = true,
	},
	buffBox = { 
		x = 100,
		y = -150,	
		--key = 'F',
		enabled = true,
	},	
	sealBox = {
		x = 150,
		y = -150,
		enabled = true,
	},
	firstLoad = true,
	boxBorder = true,
	boxLock = false,	
	chatMsg = true,
	tooltip = true,
	pallypower = true,
	earlyRebuff = 30,
	-- Menu Options, for later Use maybe
	menu = {	
		bok = true,
		bosa = true,	
	},
	fu = {
		buff = true,
		greater = true,
		need = true,
		ignore = true,
		text = "B:%b F: %f N: %n; %s",
	},	
} ) 

CPala.cmdBuffList = { "BoM", "BoK", "BoL", "BoW", "BoS", "BoSA", "GBoM", "GBoK", "GBoL", "GBoW", "GBoS", "GBoSA", CPala.none}

-- short names always better
CPala.shortcuts = {
	["Blessing of Might"] = "BoM",
	["Blessing of Kings"] = "BoK",
	["Blessing of Light"] = "BoL",
	["Blessing of Wisdom"] = "BoW",
	["Blessing of Salvation"] = "BoS",
	["Blessing of Sanctuary"] = "BoSA",
	["Blessing of Freedom"] = "Free",
	["Blessing of Protection"] = "Prot",
	["Blessing of Sacrifice"] = "Sacri",
	["Righteous Fury"] = "Fury",  
	["Greater Blessing of Might"] = "GBoM",
	["Greater Blessing of Kings"] = "GBoK",
	["Greater Blessing of Light"] = "GBoL",
	["Greater Blessing of Wisdom"] = "GBoW",
	["Greater Blessing of Salvation"] = "GBoS",
	["Greater Blessing of Sanctuary"] = "GBoSA",
	["Judgement of the Crusader"] = "JotC",
	["Judgement of Wisdom"] = "JoW",
	["Judgement of Light"] = "JoL",
	["Judgement of Justice"] = "JoJ",
	["Hammer of Justice"] = "HoJ",
	["Holy Vengeance"] = "HV",
	[CPala.none] = CPala.none,
}

-- ***** Init Stuff *******
--Things that only be need to be one time
function CPala:OnInitialize()

	local db = self.db.char; -- Local Copy

	--Disable Code
	if db.firstLoad then
		local _, class = UnitClass("player");	
		if class ~= "PALADIN" then		
			self:Print("This is a Paladin only Addon! ... Disabling self. (If you Re-Enable this will not occur again)");
			self:ToggleActive(false);
			CPalaFu:ToggleActive(false);
		
			local num = GetNumAddOns();
			local name;
		
			for i = 1, num do
				name = GetAddOnInfo(i);
				if strsub(name, 1, 5) == "CPala" then
					DisableAddOn(i);
					break;
				end;
			end;
		
			db.firstLoad = false;
			return; -- quit init
		end;
	end;
		
	self.cbuffcmd.handler = self.cbuffcmd;	 -- we dont want the profile option ...
	self:RegisterChatCommand({"/cbuff", "/cpala"}, self.cbuffcmd);
	self.cbuffcmd.handler = self;
	
	--create all menu entrys ...
	for name in pairs(self.db.char.defaultBlessings) do
		if (name ~= self.selfbuff) then
			self:Add_Player(name, true);
		end;
	end;
	
	--Template Defaults if Nothing is Present ...
	if not self.db.char.templates then
		self.db.char.templates = {	
			["GBoK all"] = {
				["WARRIOR"] = "Greater Blessing of Kings",
				["PALADIN"] = "Greater Blessing of Kings",
				["MAGE"] = "Greater Blessing of Kings",
				["HUNTER"] = "Greater Blessing of Kings",
				["SHAMAN"] = "Greater Blessing of Kings",
				["PRIEST"] = "Greater Blessing of Kings",
				["ROGUE"] = "Greater Blessing of Kings",
				["DRUID"] = "Greater Blessing of Kings",
				["WARLOCK"] = "Greater Blessing of Kings",		
				[CPala.selfbuff] = "NONE",
			},	
			["GBoL Warrior + GBoS else"] = {
				["WARRIOR"] = "Greater Blessing of Light",
				["PALADIN"] = "Greater Blessing of Salvation",
				["MAGE"] = "Greater Blessing of Salvation",
				["HUNTER"] = "Greater Blessing of Salvation",
				["SHAMAN"] = "Greater Blessing of Salvation",
				["PRIEST"] = "Greater Blessing of Salvation",
				["ROGUE"] = "Greater Blessing of Salvation",
				["DRUID"] = "Greater Blessing of Salvation",
				["WARLOCK"] = "Greater Blessing of Salvation",		
				[CPala.selfbuff] = "NONE",
			},
			["GBoW/GBoM"] = {
				["WARRIOR"] = "Greater Blessing of Might",
				["PALADIN"] = "Greater Blessing of Wisdom",
				["MAGE"] = "Greater Blessing of Wisdom",
				["HUNTER"] = "Greater Blessing of Might",
				["SHAMAN"] = "Greater Blessing of Wisdom",
				["PRIEST"] = "Greater Blessing of Wisdom",
				["ROGUE"] = "Greater Blessing of Might",
				["DRUID"] = "Greater Blessing of Wisdom",
				["WARLOCK"] = "Greater Blessing of Wisdom",		
				[CPala.selfbuff] = "NONE",
			},			
		};
	end;
	
	--create template list ...
	for name, value in pairs(self.db.char.templates) do
		self:Add_Template(name, value);
	end
	
	
	--Init Stuff
	if db.sealBox.enabled then
		self:InitSealBox();
	end;
	--[[if db.buffTime.enabled then
		self:InitBuffTime(); -- no init needed
	end;]]
	if  db.buffBox.enabled then
		self:InitBuffNeed();
	end;

	-- Remove not acitve buffs
	if (not db.menu.bok) then	
		for k,v in pairs(self.cmdBuffList) do
			if(v == "BoK" or v == "GBoK") then
				table.remove(self.cmdBuffList, k);		
			end
		end
		self.cbuffcmd.args.templates.args.gbok = nil;
	end;
		
	if (not db.menu.bosa) then
		for k,v in pairs(self.cmdBuffList) do
			if(v == "BoSA" or v == "GBoSA") then
				table.remove(self.cmdBuffList, k);		
			end
		end
		self.cbuffcmd.args.templates.args.gbosa = nil;
	end;
	
	self:RegisterEvent("UPDATE_BINDINGS");
	
	if db.pallypower then
		self:InitPally();
	end;
end


local aura_test1, aura_test2 = false; -- save time in UNIT_AURA event ...

function CPala:OnEnable()
	local db = self.db.char; -- Local Copy
	
	-- Start Stuff
	if db.buffTime.enabled then	
		self:EnableBuffTime();
	end;
	if db.sealBox.enabled then
		self:EnableSealBox();
	end;
	if db.buffBox.enabled then
		self:EnableBuffNeed();
	end;

	if db.tooltip then
		self:EnableTooltip();
	end;	
	aura_test1 = self.db.char.sealBox.enabled;
	aura_test2 = self.db.char.buffBox.enabled;
	self:RegisterEvent("UNIT_AURA");	
	self:RegisterEvent("PLAYER_TARGET_CHANGED");	
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
		
	if db.firstLoad then
		self:Print("First Loaded, Please Set Keybindings!");
		db.firstLoad = false;
	end;
	
	if db.pallypower then
		self:EnablePally();
	end
end

function CPala:EnableTooltip()
	--Tootip Display is Used by needList AND buffList
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
	self:SecureHook(GameTooltip, "SetUnit", "UPDATE_MOUSEOVER_UNIT");	
end;

function CPala:DisableTooltip()
	--Tootip Display is Used by needList AND buffList
	self:UnregisterEvent("UPDATE_MOUSEOVER_UNIT");
	self:Unhook(GameTooltip, "SetUnit");	
end;

function CPala:OnDisable()
	self:Print("You Should not disable this (not implemented)");	
end

lastInfo = ""; --save for last data

-- Debug, Reset Data when leaving a bg

function CPala:PLAYER_ENTERING_WORLD()
	local _, info = IsInInstance();
	
	if info ~= "pvp" and lastInfo == "pvp" then
		self:Clear();		
	end;
	lastInfo = info;
end;

function CPala:PLAYER_TARGET_CHANGED()
	if (aura_test1) then
		self:PLAYER_AURAS_CHANGED()
	end;
	if (aura_test2) then
		self:BuffNeedTargetChanged();
	end;
end;

function CPala:UNIT_AURA(unit, a1, a2, a3, a4, a5)	
	if (aura_test1) then
		self:SealBoxAura(unit);
	end
	if (aura_test2) then
		self:BuffNeedAura(unit);
	end;
end;


function CPala:UPDATE_BINDINGS()
	local db = self.db.char; -- Local Copy	
	if InCombatLockdown() then
		--canot update during combat!
		self:ScheduleLeaveCombatAction(self.UPDATE_BINDINGS, self);
		return;
	end;
	--[[if db.buffTime.enabled then	
		self:SetBuffTimeKeys();
	end;]]
	--[[if db.sealBox.enabled then
		self:SetSealBoxKeys();
	end;]]
	if db.buffBox.enabled then
		self:SetBuffBoxKeys();
	end;		
end;

--**** Utility Functions ****
-- Check if the unit has a specific buff, only checks self-castable buffs ;)
function CPala:HasBuff(unit, buff, timed)
	local i = 1;	
	local buff_name, _, _, _, _, left = UnitBuff(unit, i, true);
	while (buff_name) do
		if buff_name == buff then 
			
			-- test code pre end rebuffing
			if (timed) and (left) and (left < timed) then				
				return false;
			end;
		
		
			return true;
		end;
		i = i + 1;
		buff_name, _, _, _, _, left = UnitBuff(unit, i, true);
	end;
	return false;
end;

-- Check if the unit has a specific buff,
function CPala:HasBuffAll(unit, buff)
	local i = 1;		
	local buff_name = UnitBuff(unit, i, false);
	while (buff_name) do		
		if buff_name == buff then 			
			return true;
		end;
		i = i + 1;
		buff_name = UnitBuff(unit, i, false);
	end;
	return false;
end;

--Nice Time String
function CPala:FormatSeconds(sec)
	local minute = floor(sec / 60);
	sec = mod(sec, 60);	
	return format("%i:%.2i", minute, sec);
end;

--Print Message with check if disabled or not 
function CPala:Msg(text)
	if self.db.char.chatMsg then
		self:Print(text);
	end;
end;

--Switch a state of a state frame to update changes
-- will end up in state1, and if it start in state1 it will change to state2 before
function CPala:UpdateStateFrame(frame, state1, state2)
	if frame:GetAttribute("state") == state1 then
		frame:SetAttribute("state", state2);
	end;
	frame:SetAttribute("state", state1);
end

--******* Shared Events ******
function CPala:UPDATE_MOUSEOVER_UNIT()
	if UnitExists("mouseover") and UnitCanAssist("mouseover", "player")  then
		local entry = self.buffList[UnitName("mouseover")];
		local tip = GameTooltip;		
		if (not entry) then
			local _, class = UnitClass("mouseover");
			entry = self.buffList[class];
		end;
		if entry then
			local spell = entry[1];
			if (not entry[4]) then
				if (strfind(spell, "Greater"))  then
					if UnitPlayerOrPetInRaid("mouseover") then					
						if  (not self:HasBuff("mouseover", spell)) then
							tip:AddLine(format("%s faded, Group buff lasting %s", self.shortcuts[spell], self:FormatSeconds(entry[3] - (time() - entry[2]))));
						else
							tip:AddLine(format("%s lasting %s", self.shortcuts[spell], self:FormatSeconds(entry[3] - (time() - entry[2]))));
						end;
					end;
				else
					tip:AddLine(format("%s lasting %s", self.shortcuts[spell], self:FormatSeconds(entry[3] - (time() - entry[2]))));
				end;
			elseif (strfind(spell, "Greater")) then
				if UnitPlayerOrPetInRaid("mouseover") then											
					tip:AddLine(format("%s faded", self.shortcuts[spell]));
				end;
			else
				tip:AddLine(format("%s faded", self.shortcuts[spell]));
			end;
		end;
		entry = self.needList[UnitName("mouseover")];
		if (entry) then
			if (entry[1] ~= self.none) then
				tip:AddLine(format("Need %s", self.shortcuts[entry[1]]));
			end;
		end;
		tip:Show();
	end;
end;


--*******Chat Command *********
CPala.cbuffcmd = {
    type = 'group',
    name = 'cbuff',
    args = {
		--templates first
		templates =  {
			type = 'group',
			name = 'Templates',
			desc = 'Setup all Class Buffs in On Click (including Selfbuff)',
			order = 5,
			args = {
				header1 = {
					type = 'header',
					order = 25,
				},
				add = {
					order  = 30,
					type = 'text',
					name = 'New Template',
					desc = 'Save the Current Buff Settings as a Template',
					get = false,
					usage = '<Name for new Template>',
					set = function(name)
						if not name then
							CPala:Print("Can't Create a Template Without a Name!");
							return
						end;		
						if (CPala.cbuffcmd.args.templates.args[name]) and (CPala.cbuffcmd.args.templates.args[name].order ~= 1) then
							CPala:Print("Can't Create a Template With this Name!");
							return
						end
						CPala.db.char.templates[name] = {};
						local newTemplate = CPala.db.char.templates[name];
						local defaults = CPala.db.char.defaultBlessings;
						for class in pairs(CPala.classList) do
							newTemplate[class] = defaults[class];
						end
						newTemplate[CPala.selfbuff] = defaults[CPala.selfbuff];
						CPala:Print("Created a new Template using the current Class-Settings!");			
						CPala:Add_Template(name, newTemplate);
					end,
					
				},
				remove = {
					order  = 30,
					type = 'text',
					name = 'Remove Template',
					desc = 'Removes a Template',
					get = false,
					set = function(name) 
						if not name then
							CPala:Print("Remove Template Requies a Name!");
							return
						end;						
						CPala.db.char.templates[name] = nil;
						CPala.cbuffcmd.args.templates.args[name] = nil;
						CPala.cbuffcmd.args.templates.args.remove.validate[name] = nil;						
						CPala:Print("Removed Template " .. name .. "!");
					end,
					validate = {},
				},
				header2 = {
					type = 'header',
					order = 35,
				},
				allSmall = {
					name = 'Switch All to Small',
					order =  40,
					type = 'execute',
					desc = 'Change All Class-Buffs to 5 Minute Versions',
					func = function()
						local db = CPala.db.char.defaultBlessings;
						for class in pairs(CPala.classList) do
							if (db[class] ~= CPala.none) then
								if (strfind(db[class], "Greater")) then								
									db[class] = gsub(db[class], "Greater ", "");																
								end
							end;
						end;
						CPala:Print("Changed all Class-Buffs to 5 Minute Versions!");
					end,
				},
				allGreater = {
					name = 'Switch All to Greater',
					order =  40,
					type = 'execute',
					desc = 'Change All Class-Buffs to 15 Minute Versions',
					func = function()
						local db = CPala.db.char.defaultBlessings;
						for class in pairs(CPala.classList) do
							if (db[class] ~= CPala.none) then
								if not (strfind(db[class], "Greater")) then								
									db[class] = "Greater " .. db[class];
								end
							end;
						end;
						CPala:Print("Changed all Class-Buffs to 15 Minute Versions!");
					end,
				},
			
		
			}, 
		},
		--then all class stuff
		header0 = {
			type = 'header',
			order = 15,
		},
		--PlayerList		
		selfbuff = {
			type = 'text',
			name = CPala.selfbuff,
			desc = "Setup Default Buff for Self",
			get = function() return CPala.shortcuts[CPala.db.char.defaultBlessings[CPala.selfbuff]] end,
			set = function(msg) 
					CPala.db.char.defaultBlessings[CPala.selfbuff] = CPala:Get_By_Shortcut(msg); 
					CPala:CheckBuffTime();
					CPala:CheckBuffNeed();									
				end,
			validate = { "Fury", CPala.none },
			order = 19,
		},		
		header1 = {
			type = 'header',
			order = 25,
		},
		--Options
		add = {
			type = 'text',
			name = 'Add',
			desc = 'Add a Specific Player',			
			set = 'Add_Player',
			get = function() 
				--display current Target Name as default if it does make sense
				if UnitExists("target") and UnitPlayerOrPetInRaid("target") and (not CPala.db.char.defaultBlessings[UnitName("target")]) then
					return UnitName("target");
				end;
			end,
			usage = '<name>',
			order = 30,
		},
		remove = {
			type = 'text',
			name = 'Remove',
			desc = 'Remove a Specific Player',
			get = false,
			set = 'Remove_Player',
			usage = '<target>',
			validate = {},
			order = 30,
		},
		clear = {
			type = 'execute',
			name = 'Clear Data',
			desc = 'Clear Saved Buff Data',
			func = 'Clear',
			order = 31,
		},
		update = {
			type = 'execute',
			name = 'Update Needs',
			desc = 'Force a Need List Update',
			func = 'CheckBuffNeed',
			order = 31,
		},		
		header2 = {
			type = 'header',
			order = 35,
		},				
		options = {
			type = 'group',
			name = 'Options',
			desc = 'Set General Options',
			disabled = InCombatLockdown,
			order = 40,
			args = {
				lock = {
					order = 10,
					type = 'toggle',
					name = 'Lock',
					desc = 'Lock/Unlock Box Draging',
					get = function() return CPala.db.char.boxLock end,
					set = 'Set_Lock',
					disabled = InCombatLockdown,
				},
				border = {
					order = 11,
					type = 'toggle',
					name = 'Border',
					desc = 'Toggle Box-Border Visiblity',
					get = function() return CPala.db.char.boxBorder end,
					set = 'Set_Border',
					disabled = InCombatLockdown,
				},
				reset = {
					order = 12,
					type = 'execute',
					name = 'Reset Positions',
					desc = 'Reset Box Positions',
					func = function() 
						CPala.needbox:SetPos(100, -100);
						CPala.db.char.buffBox.x = 100;
						CPala.db.char.buffBox.y = -100;
						CPala.sealbox:SetPos(150, -150);
						CPala.db.char.sealBox.x = 150;
						CPala.db.char.sealBox.y = -150;						
					end,
				},
				header1 = {
					type = 'header',
					order = 20,
				},	
				bok = {
					order = 30,
					type = 'toggle',
					name = 'BoK',
					desc = 'Enable/Disable BoK as selecteable buff',
					get = function() return CPala.db.char.menu.bok end,
					set = function(value) 
						CPala.db.char.menu.bok = value; 
						if (value) then							
							table.remove(CPala.cmdBuffList, "BoSA");
							table.remove(CPala.cmdBuffList, "GBoSA");
						else
							for k,v in pairs(CPala.cmdBuffList) do
								if(v == "BoSA" or v == "GBoSA") then
									table.remove(CPala.cmdBuffList, k);		
								end
							end
							CPala.cbuffcmd.args.templates.args.gbosa = nil;
						end;
					end,
				},
				bosa = {
					order = 31,
					type = 'toggle',
					name = 'BoSA',
					desc = 'Enable/Disable BoSA as selecteable buff',
					get = function() return CPala.db.char.menu.bosa end,
					set = function(value) 
						CPala.db.char.menu.bosa = value; 
						if (value) then							
							table.remove(CPala.cmdBuffList, "BoK");
							table.remove(CPala.cmdBuffList, "GBoK");
						else
							for k,v in pairs(CPala.cmdBuffList) do
								if(v == "BoK" or v == "GBoK") then
									table.remove(CPala.cmdBuffList, k);		
								end
							end
							CPala.cbuffcmd.args.templates.args.gbok = nil;
						end;
					end,
				},
				chatMsg	= {						
					type = 'toggle',
					name = 'Chat Messages',
					order = 32,
					desc = 'Enable/Disable Chat Messages',
					get = function() return CPala.db.char.chatMsg end,
					set = function(value) 
						CPala.db.char.chatMsg = value; 						
					end,
				},
				tooltip	= {						
					type = 'toggle',
					name = 'Tooltip',
					order = 33,
					desc = 'Enable/Disable Tooltip Display',
					get = function() return CPala.db.char.tooltip end,
					set = function(value) 
						CPala.db.char.tooltip = value; 		
						if (value) then
							CPala:EnableTooltip();
						else
							CPala:DisableTooltip();
						end;
					end,
				},
				shorts = {						
					type = 'toggle',
					name = 'Short Blessings',
					order = 34,					
					desc = 'Enable/Disable Short Blessing Duration Chat Message (Freedom, Protection, Sacrifice)',
					get = function() return CPala.db.char.buffTime.shorts end,
					set = function(value) 
						CPala.db.char.buffTime.shorts = value; 						
					end,
				},
				pallypower= {						
					type = 'toggle',
					name = 'Emulate PallyPower',
					order = 35,					
					desc = 'Enable/Disable Short Blessing Duration Chat Message (Freedom, Protection, Sacrifice)',
					get = function() return CPala.db.char.pallypower end,
					set = function(value) 
						CPala.db.char.pallypower = value; 
						if (value) then
							CPala:InitPally();
							CPala:EnablePally();
						else
							self:UnregisterEvent("CHAT_MSG_ADDON");
						end;
					end,
				},
				earlyRebuff= {
					type = 'range',
					order = 35,
					name = 'Early Rebuff Time',
					desc = 'Seconds to rebuff before the buff expire',
					min = 0,
					max = 180,
					step = 1,
					get = function()
						return CPala.db.char.earlyRebuff;
					end,
					set = function(value)
						CPala.db.char.earlyRebuff = value;
					end,
					disabled = InCombatLockdown,
				},
				
				header2 = {
					type = 'header',
					order = 40,
				},			
				modules = {
					type = 'group',
					name = 'Module Config',
					desc = 'Enabling/Disabling Modules',					
					order = 43,
					args = {
						sealBox = {						
							type = 'toggle',
							name = 'SealBox',
							order = 5,
							desc = 'Enable/Disable SealBox (Box when No Seal, Judgement List on Target and Cleanse Reminder)',
							get = function() return CPala.db.char.sealBox.enabled end,
							set = function(value) 
								CPala.db.char.sealBox.enabled = value; 
								if (value) then
									CPala:InitSealBox();
									CPala:EnableSealBox();
								else
									CPala:Print("SealBox now inactive - need UI Reload to take effect");
								end;
							end,
						},
						buffBox = {						
							type = 'toggle',
							name = 'BuffBox',
							order = 5,
							desc = 'Enable/Disable BuffBox (Buff Need List)',
							get = function() return CPala.db.char.buffBox.enabled end,
							set = function(value) 
								CPala.db.char.buffBox.enabled = value; 
								if (value) then
									CPala:InitBuffNeed();
									CPala:EnableBuffNeed();									
								else
									CPala:Print("BuffBox now inactive - need UI Reload to take effect");
								end;
							end,
						},						
						buffTime= {						
							type = 'toggle',
							name = 'Buff Timer',
							order = 5,
							desc = 'Enable/Disable BuffTimer',
							get = function() return CPala.db.char.buffTime.enabled end,
							set = function(value) 
								CPala.db.char.buffTime.enabled = value; 
								if (value) then
									--CPala:InitBuffTime();
									CPala:EnableBuffTime();
								else
									CPala:Print("BuffTimer now inactive - need UI Reload to take effect");									
								end;
							end,
						},		
						
					},
				},
				header3 = {
					type = 'header',
					order = 45,
				},
				defaults = {
					order = 50,
					type = 'text',
					name = 'Set Defaults',
					desc = 'Return all Settings to default values, Requires Ui Reload',
					get = false,
					set = function(text)
						if (text == "OK" or text == "ok" or text == "Ok" or text == "oK") then
							CPala:ResetDB();
							CPalaFu:ResetDB();
							CPala:Print("All Settings now back to default - Please do /consle reloadui now");
						else
							CPala:Print("To Reset Settings enter OK as argument");
						end;
					end,
					usage = 'Type "OK"',					
				},
			},			
		},
		header3 = {
			type = 'header',
			order = 60,
		},
		
		
    },
}; 

function CPala:Get_By_Shortcut(buff)
	for key, short in pairs(self.shortcuts) do
		if string.lower(short) == string.lower(buff) then
			return key;
		end;
	end;
	return buff;
end;

function CPala:Add_Template(name, value)
	if not name or not value then
		return
	end;	
	local templateMenu = CPala.cbuffcmd.args.templates.args;
	templateMenu[name] = {}
	local new = templateMenu[name];	
	new["order"] = 1;
	new["type"] = 'execute';
	new["name"] = name;
	new["desc"] = "Use Template " .. name;
	new["func"] = function()
		local defaultBlessings = self.db.char.defaultBlessings;
		for key, buff in pairs(value) do
			defaultBlessings[key] = buff;
			CPala:PallyReportChange(key, buff) --Addon Channel Message ...
		end;
		CPala:Print("Set Classbuffs by Template " .. name .. "!");
	end;
	
	--Create Remove Entry
	templateMenu.remove.validate[name] = name;
end;

-- Adds a Special entry to the defaultBlessing list, option silent is used for default entrys during startup
function CPala:Add_Player(name, silent)
	if name then
		local defaultBlessings = self.db.char.defaultBlessings;
		local buffList= self.buffList
		if self.cbuffcmd.args[name] then			
			self:Print("Cant Add Player/Menu Entry " .. name .. " because it already exists!");
			return;
		end;
		local obj;
		if (not silent) then		
			obj = roster:GetUnitObjectFromName(name);
			if (not obj) then
				obj = roster:GetUnitObjectFromName(string.lower(name));
			end;
			if (not obj) then
				obj = roster:GetUnitObjectFromName(format("%s%s", string.upper(string.sub(name, 1, 1)), string.sub(name, 2)));
			end
		end;
		
		if ((not obj) and (not silent)) then
			self:Print(format("Cant find %s in your group/raid, maybe wrong spelling/case?", name));			
		end;
	
		-- Create Blessing List entry
		if (not silent) and (obj) then
			name = obj.name --get real name off list
		end;
		if (not silent) then
			defaultBlessings[name] = self.none;			
		end;
			
		-- create new menu entry
		local menu;
		if (self.classList[name]) then
			self.cbuffcmd.args[self.classList[name]] = {};
			menu = self.cbuffcmd.args[self.classList[name]];
		else
			self.cbuffcmd.args[name] = {};
			menu = self.cbuffcmd.args[name];
		end;		
		menu["type"] = 'text';
		if self.greaterCount[name] then
			-- Make the Class Name Look Better
			menu["name"] = self.classList[name]; --strsub(name, 1,1) .. string.lower(strsub(name,2));
		else
			menu["name"] = name;
		end;
		menu["desc"] = "Set Default Buff for " .. menu["name"];
		menu["get"] = function() return CPala.shortcuts[defaultBlessings[name]] end;
		if self.greaterCount[name] then
			menu["set"] = function(msg) 			
				local buff = CPala:Get_By_Shortcut(msg);
				defaultBlessings[name] = buff; 
				
				if buffList[name] then
					-- fake faded old buff
					if buffList[name][1] ~= buff then
						buffList[name][4] = true
						buffList[name][1] = self.none;
					end;
				end; 
				
				local key, entry, obj;
				for key, entry in pairs(buffList) do					
					obj = roster:GetUnitObjectFromName(key);
					if (obj) and (obj.class == name) then
						if buffList[obj.name] then
							-- fake faded old buff
							if buffList[obj.name][1] ~= buff then
								buffList[obj.name][4] = true
								buffList[obj.name][1] = self.none;
							end;
						end; 
					end;
				end;
				
				CPala:CheckBuffTime();
				CPala:CheckBuffNeed(); 				
				CPala:PallyReportChange(name, buff) --Addon Channel Message ...								
			end;
		else
			menu["set"] = function(msg) 			
				defaultBlessings[name] = CPala:Get_By_Shortcut(msg); 
				if buffList[name] then
					-- fake faded old buff
					if buffList[name][1] ~= defaultBlessings[name] then
						buffList[name][4] = true
						buffList[name][1] = self.none;
					end;
				end; 
				CPala:CheckBuffTime();
				CPala:CheckBuffNeed();				
			end;
		end;
		menu["validate"] = self.cmdBuffList;
			
		if (not self.greaterCount[name]) then
			menu["order"] = 20;
			tinsert(self.cbuffcmd.args.remove.validate, name);
		else
			menu["order"] = 10;
		end;
			
		if (not silent) then
			self:Print(format("Added Player %s  buff set to None", name));
		end
	else		
		self:Print("Usage : /cbuff add <target>");		
	end;
end;

function CPala:Remove_Player(name)	
	local defaultBlessings = self.db.char.defaultBlessings;
	if defaultBlessings[name] then
		-- Remove Data and Menu Entry
		local args = self.cbuffcmd.args;
		defaultBlessings[name] = nil;
		args[name] = nil;
		-- Remove Remove Entry ;)
		for k,v in pairs(args.remove.validate) do
			if(v == name) then
				table.remove(args.remove.validate, k);		
			end
		end
		self:Print("Removed entry for " .. name);
	else
		self:Print("Can Not Remove because not found;)");
	end;
end;

function CPala:Clear()
	self.buffList = {};
	self.needList = {};
	self:CheckBuffNeed();
end;

function CPala:Set_Lock(value)
	self.db.char.boxLock = value;
	if (value) then
		self.needbox:Lock();
		self.sealbox:Lock();		
	else
		self.needbox:Unlock();
		self.sealbox:Unlock();	
	end;
end;

function CPala:Set_Border(value)
	self.db.char.boxBorder = value;
	self.needbox:SetBorderVisible(value);	
	self.sealbox:SetBorderVisible(value);	
end;